"""This is the model module.

This module implements and exposes a Model class which defines the training logic and
inference logic.
"""

from .model import NewModel

# Reference your model class and name it "model"
model = NewModel
