#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""[summary]
"""


class SubmitContext:
    """[summary]"""

    def submit_model(self):
        """[summary]

        Raises:
            NotImplementedError: [description]
        """
        raise NotImplementedError

    def wait_for_completion(self):
        """[summary]

        Raises:
            NotImplementedError: [description]
        """
        raise NotImplementedError

    def download_files(self):
        """[summary]

        Raises:
            NotImplementedError: [description]
        """
        raise NotImplementedError
