# Project
PROJECT_NAME = "NAME"
SUBSCRIPTION_ID = "SUBSCRIPTION-ID"
RESOURCE_GROUP = "RESOURCE-GROUP"
WORKSPACE_NAME = "WORKSPACE-NAME"
VNET = "VNET"
SUBNET = "SUBNET"
CLUSTER_NAME = "CLUSTER-NAME"


# Model
MODEL_NAME = "NAME"
EXPERIMENT = "EXPERIMENT"
COMPUTE_TARGET = "COMPUTE-TARGET"
VM_SIZE = "VM-SIZE"
DATASETS = ["iris", "hades:2"]
DATASETS_LOCAL = ["local:2"]
DATASETS_CLOUD = ["cloud:3"]
FEATURES = ["feature1", "feature2"]
PARAMETERS = {"param1": "value1", "param2": "value2"}
FILES_INCLUDE = ["__init__.py", "__main__.py", "model.json", "model.py"]

RUN_ID = "0123456789"
EXPERIMENT_NAME = "experiment-name"

# This satisfies SonarCloud
JSON_FILE_NAME = "something.json"
VALID_JSON = '{"valid": "json"}'
