#!/usr/bin/env python
# -*- coding: utf-8 -*-
""" Azure specific implementations
"""
from energinetml.azure.backend import AzureBackend
from energinetml.azure.interactive import AzureInteractiveTrainingContext
